__all__ = ['JMan', 'JManFb', 'JManError', 'JManErrorNoContent']

import fdb
from . import fb_consultes
from . import accions
from .errors import *
from .connexio import JManCon, ConfigJavaman


class JMan:
    __slots__ = '_con'

    def __init__(self, config_data: dict):
        self._con = JManCon(config_data=config_data)

    @property
    def config(self) -> ConfigJavaman:
        return self._con.config

    @property
    def clients(self):
        return accions.Clients(con=self._con)

    @property
    def tercers(self):
        return accions.Tercers(con=self._con)

    @property
    def repartidors(self):
        return accions.Repartidors(con=self._con)

    @property
    def articles(self):
        return accions.Articles(con=self._con)

    @property
    def usuaris(self):
        return accions.Usuaris(con=self._con)

    @property
    def pobles(self):
        return accions.Pobles(con=self._con)

    @property
    def provincies(self):
        return accions.Provincies(con=self._con)

    @property
    def comandes(self):
        return accions.Comandes(con=self._con)

    @property
    def regularitzacions(self):
        return accions.Regularitzacions(con=self._con)

    @property
    def portals_web(self):
        return accions.PortalsWeb(con=self._con)

    @property
    def wms(self):
        return accions.Wms(con=self._con)

    @property
    def magatzems(self):
        return accions.Magatzems(con=self._con)

    @property
    def albarans(self):
        return accions.Albarans(con=self._con)

    @property
    def factures(self):
        return accions.Factures(con=self._con)

    @property
    def recepcions(self):
        return accions.Recepcions(con=self._con)

    @property
    def tarifes(self):
        return accions.Tarifes(con=self._con)

class JManFb:
    __slots__ = '_con'

    def __init__(self, config_data: dict):
        self._con = fdb.connect(
            user=config_data["user"],
            password=config_data["password"],
            host=config_data["host"],
            port=config_data["port"],
            database=config_data["database"],
            charset=config_data["charset"],
        )

    def select(self, sql: str, args: tuple = None) -> list:
        cur = self._con.cursor()
        cur.execute(operation=sql, parameters=args)
        res = cur.fetchall()
        cur.close()
        return res

    def c_sync_clients(self):
        return self.select(sql=fb_consultes.sync_clients)

    def c_sync_proveidors(self):
        return self.select(sql=fb_consultes.sync_proveidors)
    
    def c_sync_agents(self):
        return self.select(sql=fb_consultes.sync_empleats)

    def c_sync_clients_mrest(self):
        return self.select(sql=fb_consultes.sync_clients_mrest)

    def c_sync_productes(self):
        return self.select(sql=fb_consultes.sync_productes)

    def c_sync_condicions_pagament(self):
        return self.select(sql=fb_consultes.sync_condicions_pagament)

    def c_sync_transportistes(self):
        return self.select(sql=fb_consultes.sync_transportistes)

    def c_sync_comandes_pendents(self, magatzem_id: int):
        return self.select(sql=fb_consultes.sync_comandes_pendents, args=(magatzem_id,))

    def c_sync_magatzems_ordres_pendents(self, p_magatzem_entrada_id: int):
        return self.select(sql=fb_consultes.sync_magatzems_ordres_pendents, args=(p_magatzem_entrada_id,))

    def c_sync_moviments_magatzems_pendents(self, p_magatzem_sortida_id: int):
        return self.select(sql=fb_consultes.sync_moviments_magatzems_pendents, args=(p_magatzem_sortida_id,))

    def c_sync_comanda(self, comanda_id: int):
        return self.select(sql=fb_consultes.sync_comanda, args=(comanda_id,))[0]

    def c_sync_comanda_linies(self, comanda_id: int, magatzem_id: int):
        return self.select(sql=fb_consultes.sync_comanda_linies, args=(comanda_id, magatzem_id))



from javaman.connexio import JManCon


class Recepcions:
    __slots__ = '_con'

    _url_proveidors = '/recepcions/proveidors'
    _url_proveidor = '/recepcions/proveidors/{proveidor_id}'
    _url_proveidor_articles = '/recepcions/proveidors/{proveidor_id}/articles'
    _url_post_recepcions = '/recepcions'
    _url_post_unitat_logistica = '/recepcions/{recepcio_id}/unitat_logistica'
    _url_post_finalitzar = '/recepcions/{recepcio_id}/finalitzar'

    def __init__(self, con: JManCon):
        self._con = con

    def list_proveidors(self):
        res = self._con.get(url=self._url_proveidors)
        return res.json()

    def get_proveidor(self, proveidor_id: int):
        res = self._con.get(url=self._url_proveidor.format(proveidor_id=str(proveidor_id)))
        return res.json()

    def get_articles(self, proveidor_id: int):
        res = self._con.get(url=self._url_proveidor_articles.format(proveidor_id=str(proveidor_id)))
        return res.json()

    def post_recepcio(self, recepcio: dict):
        res = self._con.post(url=self._url_post_recepcions, data=recepcio)
        return res.json()

    def post_unitat_logistica(self, recepcio_id: int,  unitat_logistica: dict):
        res = self._con.post(url=self._url_post_unitat_logistica.format(recepcio_id=recepcio_id), data=unitat_logistica)
        return res.json()

    def post_finalitzar(self, recepcio_id: int, dades_albara: dict):
        res = self._con.post(url=self._url_post_finalitzar.format(recepcio_id=recepcio_id), data=dades_albara)
        return res.json()



sync_productes = """
select
    p.EMPRESA_ID, p.ID, p.ARTICLE, p.CODI_ARTICLE, pa.ID ATRIBUT_ID,
    p.DATA_BAIXA, pm.PES PES_NET, pm.PES + pm.TARA PES_BRUT, pm.AMPLADA, pm.LLARGADA,
    pm.ALTURA, pe.EAN13_ARTICLE, 1, pe.EAN13_CAIXA, pp.UNITATS_SUP, pl.CONTROL_LOTS_VENDA,
    alf.tip_unitat_logistica_id, alf.unitat_logistica_format_id, alf.unitats_pal
from ARTICLES p
join ARTICLES_ATRIBUTS pa on p.ID = pa.ARTICLE_ID
join ARTICLES_MIDES pm on p.ID = pm.ARTICLE_ID
join ARTICLES_EANS pe on pa.ID = pe.ARTICLE_ATRIBUT_ID
join ARTICLES_PARAMETRES pp on p.ID = pp.ARTICLE_ID
left join ( select article_id,tip_unitat_logistica_id, max(unitat_logistica_format_id) unitat_logistica_format_id,max(unitats_pal) unitats_pal
               from articles_logistica_formats alf
               join tip_unitats_logistiques tu on tu.id = alf.tip_unitat_logistica_id
              where tu.palet = 'S'
               group by article_id,tip_unitat_logistica_id ) alf on alf.article_id = p.id
left join SELECT_ARTICLES_C_ESTOCS(p.ID, CURRENT_DATE) pl on p.ID = pl.ARTICLE_ID
where p.DATA_BAIXA IS NULL;
"""

sync_clients = """
select c.EMPRESA_ID, t.ID, c.ID, t.NOM_LLARG_I_COMERCIAL, c.DATA_BAIXA, LEFT(tp.IDIOMA_CODI,2)
from clients c
join tercers t on c.TERCER_ID = t.ID
join tercers_parametres tp on t.id = tp.tercer_id
"""

sync_proveidors = """
select p.EMPRESA_ID, t.ID, p.ID, t.NOM_LLARG_I_COMERCIAL, p.DATA_BAIXA
from proveidors p
join tercers t on p.TERCER_ID = t.ID
"""

sync_empleats = """
select e.EMPRESA_ID, t.ID, e.ID, t.NOM_LLARG, e.DATA_BAIXA
from empleats e 
join tercers t on e.TERCER_ID = t.ID
"""

sync_condicions_pagament = """
  select c.ID, c.condicio_pagament
    from condicions_pagament c
"""

sync_transportistes = """
select r.EMPRESA_ID, r.ID, t.NOM_LLARG
FROM REPARTIDORS r
join TERCERS t on r.TERCER_ID = t.ID
"""

sync_comanda = """
select
    c.EMPRESA_ID, 
    c.ID COMANDA_ID,
    c.NUMERO_COMANDA,
    c.REFERENCIA,
    cd.TERCER_ID CLIENT_ID,
    c.EMPLEAT_ID,
    cd.REPARTIDOR_ID,
    CASE WHEN c_cli.COMANDA_ID IS NULL THEN cd_ter.NOM_LLARG_I_COMERCIAL ELSE c_cli.TERCER END NOM_CLIENT,
    CASE WHEN c_cli.COMANDA_ID IS NULL THEN cd_carrer.CARRER ELSE c_cli.ADRECA END ADRECA,
    CASE WHEN c_cli.COMANDA_ID IS NULL THEN cd_adr.CPOSTAL ELSE c_cli.CPOSTAL END CPOSTAL,
    CASE WHEN c_cli.COMANDA_ID IS NULL THEN cd_poble.POBLE ELSE c_cli_poble.POBLE END POBLE,
    CASE WHEN c_cli.COMANDA_ID IS NULL THEN cd_comu.PAIS_CODI ELSE c_cli_comu.PAIS_CODI END CODI_PAIS,
    CASE WHEN c_cli.COMANDA_ID IS NULL THEN coalesce(c_con.mobil ,c_con.telefon) ELSE c_cli.TELEFON END TELEFON,
    CASE WHEN c_cli.COMANDA_ID IS NULL THEN c_con.email ELSE c_cli.EMAIL END MAIL,
    cd.PES_NET, cd.PES_BRUT,
    c.obs_comanda,
    cd.obs_distribucio
from comandes c
join comandes_distribucio cd on c.ID = cd.COMANDA_ID
join adreces_distribucio cd_adr on cd.ADRECA_DISTRIBUCIO_ID = cd_adr.ID
join carrers cd_carrer on cd_adr.CARRER_ID = cd_carrer.ID
join pobles cd_poble on cd_adr.POBLE_ID = cd_poble.ID
join comarques cd_coma on cd_poble.COMARCA_ID = cd_coma.ID
join provincies cd_prov on cd_coma.PROVINCIA_ID = cd_prov.ID
join comunitats cd_comu on cd_prov.COMUNITAT_ID = cd_comu.ID

join tercers cd_ter on cd.TERCER_ID = cd_ter.ID
join clients cd_cli on cd_ter.ID = cd_cli.TERCER_ID and c.EMPRESA_ID = cd_cli.EMPRESA_ID    
left join select_contactes_tercer(cd_ter.id) c_con on c_con.tercer_id = cd_ter.id
left join comandes_client c_cli on c.ID = c_cli.COMANDA_ID
left join pobles c_cli_poble on c_cli.POBLE_ID = c_cli_poble.ID
left join comarques c_cli_coma on c_cli_poble.COMARCA_ID = c_cli_coma.ID
left join provincies c_cli_prov on c_cli_coma.PROVINCIA_ID = c_cli_prov.ID
left join comunitats c_cli_comu on c_cli_prov.COMUNITAT_ID = c_cli_comu.ID
where c.ID = ?
"""

sync_comanda_linies = """
select cl_atr.ID ATRIBUT_ID
     , sum(cl.QUANTITAT) QUANTITAT
     , min(cl.NUMERO_LINIA) nl
     , min(cl.DESCRIPCIO)
     , sum(cl.QUANTITAT) QUANTITAT_PENDENT
  from comandes c
  join comandes_linies cl on c.ID = cl.COMANDA_ID
  join articles_atributs cl_atr on cl.ARTICLE_ATRIBUT_ID = cl_atr.ID
  join articles cl_art on cl_atr.ARTICLE_ID = cl_art.ID
  LEFT join select_articles_c_estocs(cl_art.ID, c.DATA_COMANDA) cl_art_stk on cl_art.ID =  cl_art_stk.ARTICLE_ID
 where cl_art_stk.CONTROL_ESTOCS = 'S' 
   and c.ID = ?
   and cl.magatzem_id = ?
group by cl_atr.id
HAVING sum(cl.QUANTITAT) > 0
"""

sync_comandes_pendents = """
SELECT C.ID, C.NUMERO_COMANDA, C.DATA_COMANDA,C.DATA_INSERCIO,C.REFERENCIA, T.NOM_LLARG_I_COMERCIAL
  FROM COMANDES C
  JOIN CLIENTS CL ON C.CLIENT_ID = CL.ID
  JOIN TERCERS T ON CL.TERCER_ID = T.ID
  JOIN COMANDES_DISTRIBUCIO CD ON C.ID = CD.COMANDA_ID
  JOIN COMANDES_LINIES CCL ON CCL.COMANDA_ID = C.ID
  left join SELECT_COMANDES_ESTATS_MAG(C.ID) CE ON C.ID = CE.COMANDA_ID
 where C.DATA_INICI IS NULL 
   AND CE.MAG_PENDENT_SERVIR = 'S' 
   AND C.DATA_ANULACIO IS NULL
   AND CCL.MAGATZEM_ID = ?
   and CD.REPARTIDOR_ID IS NOT NULL 
 group by C.ID, C.NUMERO_COMANDA, C.DATA_COMANDA,C.DATA_INSERCIO,C.REFERENCIA, T.NOM_LLARG_I_COMERCIAL;
"""

sync_magatzems_ordres_pendents = """
SELECT MO.ID,MO.NUMERO_ORDRE, MO.DATA_INSERCIO, MO.MAGATZEM_ORDRE,MO.MAGATZEM_SORTIDA_ID, MO.MAGATZEM_ENTRADA_ID ,MO.OBSERVACIONS, U.NUMERO_UNITAT
FROM MAGATZEMS_ORDRES MO
JOIN MAGATZEMS_ORDRES_DETALL MD ON MO.ID = MD.MAGATZEM_ORDRE_ID
JOIN UNITATS_LOGISTIQUES U ON U.ID = MD.UNITAT_LOGISTICA_ID
where MO.DATA_BLOQUEIG IS null
AND MO.DATA_CONFIRMACIO IS NOT null
AND MO.DATA_FINALITZACIO IS NULL
AND MO.MAGATZEM_ENTRADA_ID = ? ;
"""

sync_moviments_magatzems_pendents = """
select ma.id,ma.magatzem_origen_id,ma.magatzem_desti_id, ma.data_insercio,ma.data_peticio,ma.observacions
from moviments_magatzems ma
where ma.data_lliurament is null
and ma.magatzem_origen_id = ? ;
"""

sync_clients_mrest = """
   select cl.id client_id_client,
        cl.codi_client,
        t.nom_llarg nom_client,
        t.nom_comercial nom_comercial_client,
        t.nif nif_client,

        ga.adreca adreca_client,
        a.cpostal cpostal_client,
        a.poble_id poble_id_client,
        p.poble poble_client,
        pr.id provincia_id_client,
        pr.provincia provincia_client,

        ga2.adreca adreca_client_distibucio,
        a2.cpostal cpostal_client_distibucio,
        a2.poble_id poble_id_client_distibucio,
        p2.poble poble_client_distibucio,
        pr2.id provincia_id_client_distibucio,
        pr2.provincia provincia_client_distibucio,

        cl.data_baixa data_baixa_client,
        cl.descompte descompte_1_client,
        cl.desc_ppago descompte_2_client,
        cl.cond_pag_id cond_pag_id_client,
        clp.recarrec recarrec_client,
        impostos.impost_normal,
        v.empleat_id empleat_id_client,

        case when cc.tercer_id is not null then cl1.id else cl.id end  client_id_destinatari,
        case when cc.tercer_id is not null then cl1.codi_client else cl.codi_client end codi_client_destinatari,
        case when cc.tercer_id is not null then t1.nom_llarg else  t.nom_llarg end nom_destinatari,
        case when cc.tercer_id is not null then t1.nom_comercial else t.nom_comercial end  nom_comercial_destinatari,
        case when cc.tercer_id is not null then t1.nif else t.nif end  nif_destinatari,

        case when cc.tercer_id is not null then cc.desc_seccio else 'N' end desc_seccio,
        case when cc.tercer_id is not null then cc.cond_pag_seccio else 'N' end cond_pag_seccio,

        case when cc.tercer_id is not null then ga1.adreca else ga.adreca end adreca_destinatari,
        case when cc.tercer_id is not null then a1.cpostal else a.cpostal end  cpostal_destinatari,
        case when cc.tercer_id is not null then a1.poble_id else a.poble_id  end  poble_id_destinatari,
        case when cc.tercer_id is not null then p1.poble else p.poble end  poble_destinatari,
        case when cc.tercer_id is not null then pr1.id  else pr.id end provincia_id_destinatari,
        case when cc.tercer_id is not null then pr1.provincia else pr.provincia end  provincia_destinatari,
        case when cc.tercer_id is not null then cl1.data_baixa else cl.data_baixa end  data_baixa_destinatari,
        case when cc.tercer_id is not null then cl1.descompte else cl.descompte end  descompte_1_destinatari,
        case when cc.tercer_id is not null then cl1.desc_ppago else cl.desc_ppago end  descompte_2_destinatari,
        case when cc.tercer_id is not null then cl1.cond_pag_id else cl.cond_pag_id end  cond_pag_id_destinatari,
        case when cc.tercer_id is not null then clp1.recarrec else clp.recarrec end  recarrec_destinatari,
        case when cc.tercer_id is not null then ct1.empleat_id else ve.id end empleat_id_destinatari,
        case when cc.tercer_id is not null then v1t.nom_llarg else vt.nom_llarg end empleat_nom_destinatari,
        v1e.data_baixa,
        vt.nom_llarg empleat_nom_client,
        ve.data_baixa

        from clients cl
        join clients_parametres clp on cl.id = clp.client_id
        join tercers t on t.id = cl.tercer_id

        join adreces a on t.id = a.tercer_id and a.id = cl.adreca_id
        join pobles p on p.id = a.poble_id
        join comarques co on co.id = p.comarca_id
        join provincies pr on pr.id = co.provincia_id
        join comunitats com on com.id = pr.comunitat_id
        join paisos pa on pa.codi = com.pais_codi
        join tip_adreces ta on ta.id = a.tip_adreca_id

        join adreces a2 on  t.id = a.tercer_id and a2.id = cl.adreca_distri_id
        join pobles p2 on p2.id = a2.poble_id
        join comarques co2 on co2.id = p2.comarca_id
        join provincies pr2 on pr2.id = co2.provincia_id
        join comunitats com2 on com2.id = pr2.comunitat_id
        join paisos pa2 on pa2.codi = com2.pais_codi
        join tip_adreces ta2 on ta2.id = a2.tip_adreca_id

        join cartera_tercers ct on ct.tercer_id = t.id and ct.empresa_id = cl.empresa_id
        join venedors v on v.empleat_id = ct.empleat_id
        join empleats ve on ct.empleat_id = ve.id
        join tercers vt on ve.tercer_id = vt.id

        left join genera_nom_adreca(ta.nom_breu,a.adreca,a.numero,a.escala,a.pis,a.porta) ga on 1 = 1
        left join genera_nom_adreca(ta2.nom_breu,a2.adreca,a2.numero,a2.escala,a2.pis,a2.porta) ga2 on 1 = 1
        left join clients_seccions cc on cc.client_id = cl.id
        left join tercers t1 on t1.id = cc.tercer_id
        left join clients cl1 on t1.id = cl1.tercer_id and cl1.empresa_id = cl.empresa_id
        left join clients_parametres clp1 on cl1.id = clp1.client_id
        left join adreces a1 on a1.id = cl1.adreca_distri_id
        left join pobles p1 on p1.id = a1.poble_id
        left join comarques co1 on co1.id = p1.comarca_id
        left join provincies pr1 on pr1.id = co1.provincia_id
        left join comunitats com1 on com1.id = pr1.comunitat_id
        left join paisos pa1 on pa1.codi = com1.pais_codi
        left join tip_adreces ta1 on ta1.id = a1.tip_adreca_id
        left join genera_nom_adreca(ta1.nom_breu,a1.adreca,a1.numero,a1.escala,a1.pis,a1.porta) ga1 on 1 = 1
        left join cartera_tercers ct1 on ct1.tercer_id = t1.id and ct1.empresa_id = cl1.empresa_id
        left join venedors v1 on v1.empleat_id = ct1.empleat_id
        left join empleats v1e on ct1.empleat_id = v1e.id
        left join tercers v1t on v1e.tercer_id = v1t.id
        left join (
            select ci.client_id,ic.normal impost_normal
            from impostos_clients ic
            join clients_impostos ci on ci.impost_id = ic.id
            where ci.data_activacio = (
                select max(data_activacio)
                from clients_impostos ci2
                where ci2.client_id = ci.client_id
                and ci2.data_activacio <= current_date 
            )
        ) impostos on impostos.client_id = cl.id
        where cl.empresa_id = 1 
        and not exists (
            select null
            from clients_seccions cs
            where cs.empresa_id = cl.empresa_id
            and cs.tercer_id = t.id
        )
        and char_length(t.nif)> 5
        and ( (t1.id is null) or ((t1.id is not null) and (v1.empleat_id is not null)) )
        order by cl.codi_client;
    """



from javaman import JManCon


class CondicionsPagament:
    __slots__ = '_con'

    _url_get_condicions = '/condicions_pagament'
    _url_get_formes_pagament = '/formes_pagament'

    def __init__(self, con: JManCon):
        self._con = con

    def get_condicions_pagament(self):
        req = self._con.get(url=self._url_get_condicions)
        return req.json()

    def get_formes_pagament(self):
        req = self._con.get(url=self._url_get_formes_pagament)
        return req.json()



from javaman.connexio import JManCon


class PortalsWeb:
    __slots__ = '_con'

    _url_portal = '/portals_web/{num_portal}'

    def __init__(self, con: JManCon):
        self._con = con

    def get_portal_web(self, portal_web_id: int):
        tmp_url = PortalsWeb._url_portal.format(num_portal=portal_web_id)
        req = self._con.get(url=tmp_url)
        return req.json()



from javaman.connexio import JManCon


class Factures:
    __slots__ = '_con'

    _url_get_llistat_factures = '/factures'
    _url_get_factures_imprimir = '/factures/{factura_id}/imprimir'

    def __init__(self, con: JManCon):
        self._con = con

    def get_factures_imprimir(self, p_factura_id: int):
        req = self._con.get(url=self._url_get_factures_imprimir.format(factura_id=p_factura_id))
        return req.json()

    def get_factures_llistat(self,
                           exercici: int = None,
                           client_id: int = None,
                           tercer_distribucio_id: int = None,
                           data_factura_inicial: str = None,
                           data_factura_final: str = None):
        tmp_url = self._url_get_llistat_factures
        tmp_params = []
        if exercici is not None:
            tmp_params.append("exercici=" + str(exercici))
        if client_id is not None:
            tmp_params.append("client_id=" + str(client_id))
        if tercer_distribucio_id is not None:
            tmp_params.append("tercer_destinatari_id=" + str(tercer_distribucio_id))
        if data_factura_inicial is not None:
            tmp_params.append("data_factura_inicial=" + data_factura_inicial)
        if data_factura_final is not None:
            tmp_params.append("data_factura_final=" + data_factura_final)
        tmp_query = ""
        if len(tmp_params) > 0:
            tmp_query = "?"
            for xx in tmp_params:
                tmp_query += "&" + xx

        req = self._con.get(url=tmp_url + tmp_query)
        return req.json()



from javaman.connexio import JManCon


class Provincies:
    __slots__ = '_con'

    _url_provincies = '/provincies'
    _url_provincies_ex = '/provincies/ex'

    def __init__(self, con: JManCon):
        self._con = con

    def list(self):
        res = self._con.get(url=self._url_provincies)
        return res.json()

    def get(self, provincia_id: int):
        res = self._con.get(url=self._url_provincies_ex + '/' + str(provincia_id))
        return res.json()



import json
import requests
from .errors import *


class ConfigJavaman:
    __slots__ = '_config'

    def __init__(self, config_data: dict):
        self._config = config_data

    def url(self):
        return self._config["url"]

    def instancia_guid(self):
        return self._config["instancia_guid"]

    def empresa_guid(self):
        return self._config["empresa_guid"]

    def usuari_manager(self):
        return self._config["usuari_manager"]

    def password_manager(self):
        return self._config["password_manager"]

    def maquina_manager(self):
        return self._config["maquina_manager"]

    def portal_web_order(self):
        return self._config["portal_web_order"]

    def portal_web_gosbi_store(self):
        return self._config["portal_web_gosbi_store"]

    def portal_web_natsbi_store(self):
        return self._config["portal_web_natsbi_store"]

    def erp_id(self):
        return self._config["erp_manager_id"]

    def magatzem_silo_id(self):
        return self._config["magatzem_silo_id"]

    def magatzem_gdis_id(self):
        return self._config["magatzem_gdis_id"]

    def magatzem_entrades_id(self):
        return self._config["magatzem_entrades_id"]

    def magatzem_santa_llogaia_id(self):
        return self._config["magatzem_santa_llogaia_id"]

    def magatzem_botiga_id(self):
        return self._config["magatzem_botiga_id"]

    def magatzem_envasat_id(self):
        return self._config["magatzem_envasat_id"]

    def tipus_unitat_logistica_id_palet(self):
        return self._config["unitat_logistica_tipus_palet_id"]

    def tipus_unitat_logistica_id_caixa(self):
        return self._config["unitat_logistica_tipus_caixa_id"]


class JManCon:
    __slots__ = '_usuari_token', '_config'

    def __init__(self, config_data: dict):
        self._config = ConfigJavaman(config_data=config_data)
        data = {
            "user_login": self._config.usuari_manager(),
            "user_password": self._config.password_manager(),
            "instancia_guid": self._config.instancia_guid(),
            "empresa_guid": self._config.empresa_guid()
        }
        headers = {
            'Content-type': 'application/json',
            'Accept': 'application/json'
        }
        req = requests.post(url=self._config.url() + '/login/usuari', headers=headers, data=json.dumps(data))
        if req.status_code not in [200, 201]:
            raise JManErrorUnauthorized()
        self._usuari_token = req.json()['token']

    @property
    def config(self) -> ConfigJavaman:
        return self._config

    @property
    def usuari_token(self) -> str:
        return self._usuari_token

    def post(self, url: str, data: [list, dict] = None) -> requests:
        headers = {
            'Accept': 'application/json',
            'authorization': "Bearer " + self.usuari_token
        }
        if data is not None:
            headers['Content-type'] = 'application/json'
        if data is not None:
            data = json.dumps(data)
        try:
            req = None
            req = requests.post(url=self._config.url() + url, headers=headers, data=data)
        except requests.exceptions.RequestException:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or 300 <= req.status_code < 400:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or req.status_code >= 500:
            raise JManErrorApp(req.content)
        if 400 <= req.status_code < 500:
            raise JManErrorClient(req.content)
        return req

    def put(self, url: str, data: [list, dict, None]) -> requests:
        headers = {
            'Accept': 'application/json',
            'authorization': "Bearer " + self.usuari_token
        }
        if data is not None:
            headers['Content-type'] = 'application/json'
        if data is not None:
            data = json.dumps(data)
        try:
            req = None
            req = requests.put(url=self._config.url() + url, headers=headers, data=data)
        except requests.exceptions.RequestException:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or 300 <= req.status_code < 400:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or req.status_code >= 500:
            raise JManErrorApp(req.content)
        if 400 <= req.status_code < 500:
            raise JManErrorClient(req.content)
        return req

    def get(self, url: str) -> requests:
        headers = {
            'Accept': 'application/json',
            'authorization': "Bearer " + self.usuari_token
        }
        try:
            req = None
            req = requests.get(url=self._config.url() + url, headers=headers)
        except requests.exceptions.RequestException:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or 300 <= req.status_code < 400:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or req.status_code >= 500:
            raise JManErrorApp(req.content)
        if 400 <= req.status_code < 500:
            raise JManErrorClient(req.content)
        return req

    def delete(self, url: str) -> requests:
        headers = {'authorization': "Bearer " + self.usuari_token}
        try:
            req = None
            req = requests.delete(url=self._config.url() + url, headers=headers)
        except requests.exceptions.RequestException:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or 300 <= req.status_code < 400:
            raise JManErrorConnection(req.content)
        if req.status_code < 200 or req.status_code >= 500:
            raise JManErrorApp(req.content)
        if 400 <= req.status_code < 500:
            raise JManErrorClient(req.content)
        return req

    def __del__(self):
        self.delete(url='/logout')



from javaman.connexio import JManCon


class Repartidors:
    __slots__ = '_con'

    _url_get_repartiors = '/repartidors/ex'
    _url_get_repartior = '/repartidors/ex/{id_rep}'
    _url_get_repartior_zones = '/repartidors/{id_rep}/zones'
    _url_get_repartior_zona = '/repartidors/{id_rep}/zona'

    def __init__(self, con: JManCon):
        self._con = con

    def get_repartidors(self):
        req = self._con.get(url=Repartidors._url_get_repartiors)
        return req.json()

    def get_repartidor(self, p_repartidor: int):
        req = self._con.get(url=Repartidors._url_get_repartior.format(id_rep=p_repartidor))
        return req.json()

    def get_repartidor_zones(self, p_repartidor: int):
        tmp_url = Repartidors._url_get_repartior_zones.format(id_rep=p_repartidor)
        req = self._con.get(url=tmp_url)
        return req.json()

    def get_repartidor_zona_by_cp(self, p_repartidor: int, p_pais_codi: str, p_codi_postal: str):
        tmp_url = Repartidors._url_get_repartior_zona.format(id_rep=p_repartidor)
        tmp_url += "?pais_codi=" + p_pais_codi + "&cpostal=" + p_codi_postal
        req = self._con.get(url=tmp_url)
        return req.json()



from javaman.connexio import JManCon


class Tarifes:
    __slots__ = '_con'

    _url_get_tip_tarifes = '/tip_tarifes'
    _url_get_tarifes = '/tip_tarifes/{tip_tarifa_id}/ex'

    def __init__(self, con: JManCon):
        self._con = con

    def get_tip_tarifes(self):
        req = self._con.get(url=self._url_get_tip_tarifes)
        return req.json()

    def get_tarifes(self, p_tip_tarifa_id: int):
        req = self._con.get(url=self._url_get_tarifes.format(tip_tarifa_id=p_tip_tarifa_id))
        return req.json()



from javaman.connexio import JManCon


class Albarans:
    __slots__ = '_con'

    _url_get_llistat_albarans = '/albarans'
    _url_get_albara_imprimir = '/albarans/{albara_id}/imprimir'

    def __init__(self, con: JManCon):
        self._con = con

    def get_albarans_imprimir(self, p_albara_id: int):
        req = self._con.get(url=self._url_get_albara_imprimir.format(albara_id=p_albara_id))
        return req.json()

    def get_albarans_llistat(self,
                             exercici: int = None,
                             client_id: int = None,
                             tercer_distribucio_id: int = None,
                             data_albara_inicial: str = None,
                             data_albara_final: str = None,
                             serie_facturacio_id:int = None,
                             numero_albara_inicial: int = None,
                             numero_albara_final: int = None,
                             article_id: int = None,
                             article_atribut_id: int = None,
                             empleat_id: int = None,
                             repartidor_id: int = None,
                             article_lot_id: int = None,
                             article_lot: str = None
                             ):
        tmp_url = self._url_get_llistat_albarans
        tmp_params = []
        if exercici is not None:
            tmp_params.append("exercici="+str(exercici))
        if client_id is not None:
            tmp_params.append("client_id=" + str(client_id))
        if tercer_distribucio_id is not None:
            tmp_params.append("tercer_destinatari_id="+str(tercer_distribucio_id))
        if data_albara_inicial is not None:
            tmp_params.append("data_albara_inicial="+data_albara_inicial)
        if data_albara_final is not None:
            tmp_params.append("data_albara_final="+data_albara_final)
        if article_id is not None:
            tmp_params.append("article_id="+str(article_id))
        if article_atribut_id is not None:
            tmp_params.append("article_atribut_id="+str(article_atribut_id))
        if empleat_id is not None:
            tmp_params.append("empleat_id="+str(empleat_id))
        if repartidor_id is not None:
            tmp_params.append("repartidor_id="+str(repartidor_id))
        if article_lot_id is not None:
            tmp_params.append("article_lot_id="+str(article_lot_id))
        if article_lot is not None:
            tmp_params.append("article_lot="+article_lot)
        if numero_albara_inicial is not None:
            tmp_params.append("numero_albara_inicial="+str(numero_albara_inicial))
        if numero_albara_final is not None:
            tmp_params.append("numero_albara_final="+str(numero_albara_final))
        if serie_facturacio_id is not None:
            tmp_params.append("serie_facturacio_id="+str(serie_facturacio_id))

        tmp_query = ""
        if len(tmp_params) > 0:
            tmp_query = "?"
            for xx in tmp_params:
                tmp_query += "&" + xx

        req = self._con.get(url=tmp_url + tmp_query)
        return req.json()



from javaman.connexio import JManCon


class Tercers:
    __slots__ = '_con'

    _url_get_tercers = '/tercer'
    _url_get_tercers_parametres = '/tercer/{id_tercer}/parametres'
    _url_get_tercers_anotacions = '/tercer/{id_tercer}/anotacions'

    def __init__(self, con: JManCon):
        self._con = con

    def get_tercer(self, p_tercer: int):
        req = self._con.get(url=self._url_get_tercers+'/'+str(p_tercer))
        return req.json()

    def get_tercer_parametres(self, p_tercer: int):
        req = self._con.get(url=Tercers._url_get_tercers_parametres.format(id_tercer=p_tercer))
        return req.json()

    def get_tercer_anotacions(self, p_tercer: int):
        req = self._con.get(url=Tercers._url_get_tercers_anotacions.format(id_tercer=p_tercer))
        return req.json()



from javaman.connexio import JManCon


class Clients:
    __slots__ = '_con'

    _url_crear_client = '/clients'
    _url_get_client = '/clients'
    _url_importar_tercers = '/importar_tercers'
    _url_client_central = '/clients_seccions?tercer_seccio_id={tercer_seccio_id}'
    _url_clients_seccions = '/clients_seccions?client_central_id={client_central_id}'

    def __init__(self, con: JManCon):
        self._con = con

    def crear(self, p_client: dict):
        req = self._con.post(url=self._url_crear_client, data=p_client)
        return req.json()

    def get_client(self, p_client: int):
        req = self._con.get(url=self._url_crear_client+'/'+str(p_client))
        return req.json()

    def importar_tercers(self, p_dades: dict):
        req = self._con.post(url=self._url_importar_tercers, data=p_dades)
        return req.json()

    def get_client_central_id(self, p_tercer_seccio_id: int) -> int:
        client_id = None
        req = self._con.get(url=self._url_client_central.format(tercer_seccio_id=p_tercer_seccio_id))
        result = req.json()
        if result is not None and len(result) > 0:
            client_id = result[0]["client_id"]
        return client_id

    def get_seccions_client(self, p_client_central_id: int) -> list:
        client_id = None
        req = self._con.get(url=self._url_clients_seccions.format(client_central_id=p_client_central_id))
        result = req.json()
        return result


from javaman.connexio import JManCon


class Usuaris:
    __slots__ = '_con'

    _url_usuaris = '/usuaris'
    _url_login = '/login/usuari'

    def __init__(self, con: JManCon):
        self._con = con

    def list(self):
        res = self._con.get(url=self._url_usuaris)
        return res.json()

    def get(self, usr_guid: str):
        res = self._con.get(url=self._url_usuaris + '/' + usr_guid)
        return res.json()

    def delete_token(self, token: str):
        self._con.delete(url='/logout/' + token)
        return

    def get_current_user(self):
        res = self._con.get(url='/current_user')
        return res.json()
